/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2009 Maxim Aldanov <aldmax@mail.ru>, Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 * */

#ifndef QPREVIEW_H
#define QPREVIEW_H

#include <QAbstractScrollArea>
#include <QPrinter>
#include <dpi_metrics.h>

//#define PRINTER_CHANGE_DEVICE

class QResizeEvent;
class QPaintEvent;
class QAbstractScrollArea;
class QMouseEvent;
class QMousePressData;
class AbstractPreviewPrivate;

class AbstractPreview : public QAbstractScrollArea
{
	Q_OBJECT

friend class AbstractPreviewPrivate;

signals:
	void pageSelected(int);
	void scaleChanged(double);
	void pageFormatChanged();

public:
	enum PreviewMode {Mode_Normal, Mode_Thumbs};
	AbstractPreview(QWidget *parent, QPrinter *printer);
	AbstractPreview(QWidget *parent, QPrinter *printer, int _countPage);
	virtual ~AbstractPreview();

	QSize paperSize();

	void setInterPageSpacing(int spacing);
	int interPageSpacing();
	void setPageMargin(int margin);
	int pageMargin();

	int pageCount();

	int addPages(int count = 1);
	int insertPages(int index, int count = 1);
	int deletePages(int index, int count = 1);
	void clear();
	void setPageData(int index, QVariant data);
	QVariant pageData(int index);


	void setMargins(qreal ml, qreal mt, qreal mr, qreal mb);
	void setMarginLeft(qreal ml);
	void setMarginTop(qreal mt);
	void setMarginRight(qreal mr);
	void setMarginBottom(qreal mb);
	qreal marginLeft();
	qreal marginTop();
	qreal marginRight();
	qreal marginBottom();


	void setViewMode(AbstractPreview::PreviewMode mode);
	AbstractPreview::PreviewMode viewMode();

	inline void updatePreview() { viewport()->update(); };
	void repaintPreview();

	void setScaleStep(double step);
	double scaleStep();
	void setScaleRange(double zmin, double zmax);
	void setScaleMaximum(double zmax);
	void setScaleMinimum(double zmin);
	double scaleMaximum();
	double scaleMinimum();

public slots:
	void setScale(double scale);
	void pageSetup();
	void setupPageFormat();
	void print();

	void scaleIn();
	void scaleOut();
	void scaleOrig();
	void gotoPage(int index);

private:
	AbstractPreviewPrivate *d;
	AbstractPreviewPrivate *dv, *dp;
	void paintBorder(QPainter *p);
	QPixmap* getTile(int page, int tx, int ty);
#ifdef PRINTER_CHANGE_DEVICE
	void changeDevice(QPaintDevice*);
#endif
	void autoscaleThumbs();

protected slots:

protected:
	int sizeToCountPage();

	virtual void printPages(QPrinter *printer);
	virtual void resizeEvent(QResizeEvent *event);
	virtual void paintEvent(QPaintEvent *event);
	virtual void scrollContentsBy(int dx, int dy);

	virtual void mousePressEvent(QMouseEvent *e);
	virtual void mouseMoveEvent(QMouseEvent *e);
	virtual void mouseReleaseEvent(QMouseEvent *e);
	virtual void wheelEvent(QWheelEvent * event);

	virtual void scaleEvent(const double &scaleOld, const double &scaleNew);
	virtual void addedPages(int count, int begin);
	virtual void paintPage(QPainter *p, int numberPage, const QRect&) = 0;
	virtual void updatePageFormat() = 0;
#ifdef PRINTER_CHANGE_DEVICE
	virtual void deviceChanged(QPaintDevice*) = 0;
#endif
	virtual void clearEvent() = 0;

	QSize getPaperSize();
	QSize getPageSize();
};
#endif // QPREVIEW_H

