/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#ifndef _MAINWIDGET_H
#define _MAINWIDGET_H

#define MAINWIDGET_TAB

#ifdef MAINWIDGET_TAB
#include <QTabWidget>
#else
#include <QWidget>
class QHBoxLayout;
class QStackedLayout;

class QListWidget;
class QListWidgetItem;
#endif

#include <qpxsettings.h>
#include <device.h>

class QIODevice;

class tabDevInfo;
class tabMediaInfo;
class tabTransfer;
class tabERRC;
class tabJB;
class tabFETE;
class tabTA;
class QPxGraph;

#ifdef MAINWIDGET_TAB
class QPxMainWidget : public QTabWidget
#else
class QPxMainWidget : public QWidget
#endif
{
	Q_OBJECT
public:
	QPxMainWidget(QPxSettings *iset, devlist *idev, QWidget *p=0);
	~QPxMainWidget();
	void drawGraph(QImage& img, device *dev, int ttype, int eflags=0);

public slots:
	void clearDev();
	void clearMedia();
	void selectDevice();
	//void reconfig();
	inline void reconfig() { emit configured(); };

signals:
	void deviceSelected();
	void configured();

private:
	devlist			*devices;
	QPxSettings		*settings;

#ifndef MAINWIDGET_TAB
	QHBoxLayout		*layout;
	QListWidget		*list;
	QStackedLayout	*stack;
#endif
	tabDevInfo		*tab_DevInfo;
	tabMediaInfo	*tab_MediaInfo;
	tabTransfer		*tab_RT;
	tabTransfer		*tab_WT;
	tabERRC			*tab_ERRC;
	tabJB			*tab_JB;
	tabFETE			*tab_FETE;
	tabTA			*tab_TA;
};

#endif

