/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <QGridLayout>
#include <QLabel>
#include <QFrame>

#include <device.h>
#include <qpxgraph.h>

#include "errc_detailed.h"
#include <QDebug>

const char* label[] = {
	"BLER",
	"E11/PIE",
	"E21/PI8",
	"E31/PIF",
	"E12/POE",
	"E22/PO8",
	"E32/POF",
	"UNCR"
};

ErrcDetailedDialog::ErrcDetailedDialog(QPxSettings *iset, devlist *idev, QWidget *p, Qt::WindowFlags fl)
	: QDialog(p,fl)
{
#ifndef QT_NO_DEBUG
	qDebug("ErrcDetailedDialog()");
#endif
	devices  = idev;
	settings = iset;

	setWindowTitle("QPxTool - " + tr("Detailed Error Correction"));

	layout = new QGridLayout(this);
	layout->setMargin(3);
	layout->setSpacing(3);

	settings->loadScale("XERRC");

	graph[0] = new QPxGraph(iset, idev, "XERRC", TEST_ERRC, this);
	graph[0]->setErrcList(GRAPH_BLER, "BLER");
	layout->addWidget(graph[0], 0, 1);

	for (int i=0; i<6; i++) {
		graph[i+1] = new QPxGraph(iset, idev, "XERRC", TEST_ERRC, this);
		graph[i+1]->setErrcList( 2 << i, label[i+1]);
		layout->addWidget(graph[i+1], i/3+1, i%3);
	}

	graph[7] = new QPxGraph(iset, idev, "XERRC", TEST_ERRC, this);
	graph[7]->setErrcList(GRAPH_UNCR, "UNCR");
	layout->addWidget(graph[7], 0, 2);

	for (int i=0; i<8; i++) graph[i]->setShowSpeed(0);

	layoutt = new QGridLayout();
	layoutt->setMargin(0);
	layoutt->setHorizontalSpacing(3);
	layoutt->setVerticalSpacing(1);
	layout->addLayout(layoutt, 0, 0);

	pl_tot = new QLabel("Tot",this);
	pl_tot->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	layoutt->addWidget(pl_tot,0,1, 1,2);
	pl_max = new QLabel("Max",this);
	pl_max->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	layoutt->addWidget(pl_max,0,3);
	pl_avg = new QLabel("Avg",this);
	pl_avg->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	layoutt->addWidget(pl_avg,0,4);

	hline0 = new QFrame(this);
	hline0->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	layoutt->addWidget(hline0, 1,0, 1,5);

	for (int i=0; i<8; i++) {
		pl_name[i] = new QLabel(label[i], this);
//		pl_name[i]->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
		layoutt->addWidget(pl_name[i], i+2, 0);
		l_tot[i] = new QLabel(this);
		l_tot[i]->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
		layoutt->addWidget(l_tot[i], i+2, 1, 1,2);
		l_max[i] = new QLabel(this);
		l_max[i]->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
		layoutt->addWidget(l_max[i], i+2, 3);
		l_avg[i] = new QLabel(this);
		l_avg[i]->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
		layoutt->addWidget(l_avg[i], i+2, 4);
	}

	hline1 = new QFrame(this);
	hline1->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	layoutt->addWidget(hline1, 10,0, 1,5);

	layoutt->setRowStretch(11,10);

	for (int i=0; i<8; i++)
		connect(graph[i], SIGNAL(scaleChanged()), this, SLOT(changeScale()));
}

ErrcDetailedDialog::~ErrcDetailedDialog()
{
#ifndef QT_NO_DEBUG
	qDebug("~ErrcDetailedDialog()");
#endif
}

void ErrcDetailedDialog::changeScale()
{
	for (int i=0; i<8; i++) graph[i]->changeScale();
}

void ErrcDetailedDialog::updateAll()
{
	device *dev = devices->current();
	for (int i=0; i<8; i++) {
		l_tot[i]->setText( QString::number(dev->testData.errcTOT.raw.err[i]) );
		l_max[i]->setNum( dev->testData.errcMAX.raw.err[i] );
		l_avg[i]->setText( QString::number(dev->testData.errcAVG.raw.err[i], 'f', 2) );
		graph[i]->update();
	}
}

/*
void ErrcDetailedDialog::closeEvent(QCloseEvent* e)
{
//	emit closed();
	QDialog::closeEvent(e);
}
*/

void ErrcDetailedDialog::hideEvent(QHideEvent* e)
{
	emit closed();
	QDialog::hideEvent(e);
}

