/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2007-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#ifndef MAINWIDGET_TAB
#include <QStackedLayout>
#include <QHBoxLayout>
#include <QListWidget>
#include <QListWidgetItem>
#endif

#include <QIcon>

#include <device.h>

#include <tab_devinfo.h>
#include <tab_mediainfo.h>
#include <tab_transfer.h>
#include <tab_errc.h>
#include <tab_jb.h>
#include <tab_fete.h>
#include <tab_ta.h>

#include "mainwidget.h"

#include <QDebug>

QPxMainWidget::QPxMainWidget(QPxSettings *iset, devlist *idev, QWidget *p)
#ifdef MAINWIDGET_TAB
	: QTabWidget(p)
#else
	: QWidget(p)
#endif

{
#ifndef QT_NO_DEBUG
	qDebug("STA: QPxMainWidget()");
#endif
	settings = iset;
	devices = idev;

#ifndef MAINWIDGET_TAB
	QListWidgetItem *it;
	layout = new QHBoxLayout(this);
	layout->setMargin(3);
	layout->setSpacing(3);

	list = new QListWidget(this);
	layout->addWidget(list);

	stack = new QStackedLayout();
	layout->addLayout(stack);
#endif
	
	tab_DevInfo = new tabDevInfo(iset, idev, this);
#ifdef MAINWIDGET_TAB
	addTab(tab_DevInfo, QIcon(":images/cdwriter.png"), tr("Device"));
#else
	stack->addWidget(tab_DevInfo);
	list->addItem( new QListWidgetItem(tr("Device")) );
#endif
	connect(this, SIGNAL(deviceSelected()), tab_DevInfo, SLOT(selectDevice()));

	tab_MediaInfo = new tabMediaInfo(iset, idev, this);
#ifdef MAINWIDGET_TAB
	addTab(tab_MediaInfo, QIcon(":images/info.png"), tr("Media"));
#else
	stack->addWidget(tab_MediaInfo);
	list->addItem( new QListWidgetItem(tr("Media")) );
#endif
	connect(this, SIGNAL(deviceSelected()), tab_MediaInfo, SLOT(selectDevice()));

	tab_RT = new tabTransfer(iset, idev, "RT", 0, this);
#ifdef MAINWIDGET_TAB
	addTab(tab_RT, tr("Read Transfer"));
#else
	stack->addWidget(tab_RT);
	list->addItem( new QListWidgetItem(tr("Read Transfer")) );
#endif
	connect(this, SIGNAL(configured()), tab_RT, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_RT, SLOT(selectDevice()));

	tab_WT = new tabTransfer(iset, idev, "WT", 1, this);
#ifdef MAINWIDGET_TAB
	addTab(tab_WT, tr("Write Transfer"));
#else
	stack->addWidget(tab_WT);
	list->addItem( new QListWidgetItem(tr("Write Transfer")) );
#endif
	connect(this, SIGNAL(configured()), tab_WT, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_WT, SLOT(selectDevice()));

	tab_ERRC = new tabERRC(iset, idev, "ERRC", this);
#ifdef MAINWIDGET_TAB
	addTab(tab_ERRC, tr("Error Correction"));
#else
	stack->addWidget(tab_ERRC);
	list->addItem( new QListWidgetItem(tr("Error Correction")) );
#endif
	connect(this, SIGNAL(configured()), tab_ERRC, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_ERRC, SLOT(selectDevice()));

	tab_JB = new tabJB(iset, idev, "JB", this);
#ifdef MAINWIDGET_TAB
	addTab(tab_JB, tr("Jitter/Asymmethry"));
#else
	stack->addWidget(tab_JB);
	list->addItem( new QListWidgetItem(tr("Jitter/Asymmethry")) );
#endif
	connect(this, SIGNAL(configured()), tab_JB, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_JB, SLOT(selectDevice()));

	tab_FETE = new tabFETE(iset, idev, "FETE", this);
#ifdef MAINWIDGET_TAB
	addTab(tab_FETE, tr("FE/TE"));
#else
	stack->addWidget(tab_FETE);
	list->addItem( new QListWidgetItem(tr("FE/TE")) );
#endif
	connect(this, SIGNAL(configured()), tab_FETE, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_FETE, SLOT(selectDevice()));

	tab_TA = new tabTA(iset, idev, "TA", this);
#ifdef MAINWIDGET_TAB
	addTab(tab_TA, tr("Time Analyser"));
#else
	stack->addWidget(tab_TA);
	list->addItem( new QListWidgetItem(tr("Time Analyser")) );
#endif

	
	connect(this, SIGNAL(configured()), tab_TA, SLOT(update()));
	connect(this, SIGNAL(deviceSelected()), tab_TA, SLOT(selectDevice()));
#ifndef MAINWIDGET_TAB
	connect(list, SIGNAL(currentRowChanged(int)), stack, SLOT(setCurrentIndex(int)));
#endif

#ifndef QT_NO_DEBUG
	qDebug("END: QPxMainWidget()");
#endif
}

QPxMainWidget::~QPxMainWidget()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~QPxMainWidget()");
	qDebug("END: ~QPxMainWidget()");
#endif
}
/*
void QPxMainWidget::reconfig()
{
	emit configured();
}
*/
void QPxMainWidget::clearDev()
{
	tab_MediaInfo->clear();
	tab_DevInfo->clear();
}

void QPxMainWidget::clearMedia()
{
	tab_MediaInfo->clear();
}

void QPxMainWidget::selectDevice()
{
	emit deviceSelected();
}

void QPxMainWidget::drawGraph(QImage& img, device *dev, int ttype, int eflags)
{
	switch(ttype) {
		case TEST_RT:
			tab_RT->drawGraph(img, dev, TEST_RT);
			break;
		case TEST_WT:
			tab_WT->drawGraph(img, dev, TEST_WT);
			break;
		case TEST_ERRC:
			tab_ERRC->drawGraph(img, dev, TEST_ERRC, eflags);
			break;
		case TEST_JB:
			tab_JB->drawGraph(img, dev, TEST_JB);
			break;
		case TEST_FT:
			tab_FETE->drawGraph(img, dev, TEST_FT);
			break;
		case TEST_TA:
			tab_TA->drawGraph(img, dev, TEST_TA, eflags);
			break;
	}
}

