/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>

#include <qpxgraph.h>
#include <errc_detailed.h>
#include "tab_errc.h"
#include <QDebug>

tabERRC::tabERRC(QPxSettings *iset, devlist *idev, QString iname, QWidget *p, Qt::WindowFlags fl)
	: GraphTab(iset, idev, iname, TEST_ERRC, p, fl)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: tabERRC()");
#endif
	xerrc = NULL;

	layout_info = new QVBoxLayout(infow);
	layout_info->setMargin(0);
	layout_info->setSpacing(3);

	pl_e0 = new QLabel("BLER/PI8", infow);
	pl_e0->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_e0->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	layout_info->addWidget(pl_e0);

	l_e0t = new QLabel(infow);
	l_e0t->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e0t->setMinimumHeight(22);
	layout_info->addWidget(l_e0t);

	l_e0m = new QLabel(infow);
	l_e0m->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e0m->setMinimumHeight(22);
	layout_info->addWidget(l_e0m);

	l_e0a = new QLabel(infow);
	l_e0a->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e0a->setMinimumHeight(22);
	layout_info->addWidget(l_e0a);


	pl_e1 = new QLabel("E22/PIF", infow);
	pl_e1->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_e1->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	layout_info->addWidget(pl_e1);

	l_e1t = new QLabel(infow);
	l_e1t->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e1t->setMinimumHeight(22);
	layout_info->addWidget(l_e1t);

	l_e1m = new QLabel(infow);
	l_e1m->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e1m->setMinimumHeight(22);
	layout_info->addWidget(l_e1m);

	l_e1a = new QLabel(infow);
	l_e1a->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e1a->setMinimumHeight(22);
	layout_info->addWidget(l_e1a);


	pl_e2 = new QLabel("E32/POF", infow);
	pl_e2->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_e2->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	layout_info->addWidget(pl_e2);

	l_e2t = new QLabel(infow);
	l_e2t->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e2t->setMinimumHeight(22);
	layout_info->addWidget(l_e2t);

	l_e2m = new QLabel(infow);
	l_e2m->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e2m->setMinimumHeight(22);
	layout_info->addWidget(l_e2m);

	l_e2a = new QLabel(infow);
	l_e2a->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_e2a->setMinimumHeight(22);
	layout_info->addWidget(l_e2a);


#ifdef ENABLE_UNCR
	pl_u = new QLabel("UNCR", infow);
	pl_u->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_u->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	layout_info->addWidget(pl_u);

	l_ut = new QLabel(infow);
	l_ut->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_ut->setMinimumHeight(22);
	layout_info->addWidget(l_ut);

	l_um = new QLabel(infow);
	l_um->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_um->setMinimumHeight(22);
	layout_info->addWidget(l_um);

	l_ua = new QLabel(infow);
	l_ua->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_ua->setMinimumHeight(22);
	layout_info->addWidget(l_ua);
#endif


	layout_info->addStretch(10);

	pb_xerrc = new QPushButton("Detailed");
	layout_info->addWidget(pb_xerrc);

	QObject::connect(pb_xerrc, SIGNAL(clicked()), this, SLOT(toggleXErrc()));

	clear();
#ifndef QT_NO_DEBUG
	qDebug("END: tabERRC()");
#endif
}

tabERRC::~tabERRC()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~tabERRC()");
	qDebug("END: ~tabERRC()");
#endif
}
/*
void tabERRC::clear()
{
	qDebug("tabERRC::clear()");
}
*/
void tabERRC::selectDevice()
{
#ifndef QT_NO_DEBUG
	qDebug("tabERRC::selectDevice()");
#endif
	device *dev = devices->current();
	updateAll();
//	GraphTab::updateLast((int)time, NULL, 1);
//	updateSummary(dev);

	//	QObject::connect( devices->current(), SIGNAL(block_ERRC()), this, SLOT(update()) );
	QObject::connect( dev, SIGNAL(doneMInfo(int)), this, SLOT(updateAll()) );
	QObject::connect( dev, SIGNAL(block_ERRC()), this, SLOT(updateLast()) );
}

void tabERRC::updateAll()
{
	device *dev = devices->current();
	GraphTab::updateLast((int)(dev->testData.errc_time), NULL, 1);
	updateSummary(dev);
	if (dev->media.type.startsWith("CD")) {
		pl_e0->setText("BLER");
		pl_e1->setText("E22");
		pl_e2->setText("E32");
	} else if (dev->media.type.startsWith("DVD")) {
		pl_e0->setText("PI8");
		pl_e1->setText("PIF");
		pl_e2->setText("POF");
	}
}

void tabERRC::updateLast()
{
	bool show;
	device *dev = devices->current();
	int  time = (int) (dev->testData.errc_time);
	GraphTab::updateLast(time, &show);
	if (!show) return;
	updateSummary(dev);

	if (xerrc && xerrc->isVisible()) xerrc->updateAll();
}

void tabERRC::updateSummary(device *dev)
{
	if (!dev->testData.errc.size()) {
		l_e0t->clear(); l_e0m->clear(); l_e0a->clear(); 
		l_e1t->clear(); l_e1m->clear(); l_e1a->clear(); 
		l_e2t->clear(); l_e2m->clear(); l_e2a->clear(); 
#ifdef ENABLE_UNCR
		l_ut->clear();  l_um->clear();  l_ua->clear();
#endif
		return;
	}
	if (dev->media.type.startsWith("CD")) {
		l_e0t->setText( QString::number(dev->testData.errcTOT.cd.bler) );
		l_e0m->setNum( dev->testData.errcMAX.cd.bler );
		l_e0a->setText( QString::number(dev->testData.errcAVG.cd.bler, 'f', 2) );

		l_e1t->setText( QString::number(dev->testData.errcTOT.cd.e22) );
		l_e1m->setNum( dev->testData.errcMAX.cd.e22 );
		l_e1a->setText( QString::number(dev->testData.errcAVG.cd.e22, 'f', 2) );

		l_e2t->setText( QString::number(dev->testData.errcTOT.cd.e32) );
		l_e2m->setNum( dev->testData.errcMAX.cd.e32 );
		l_e2a->setText( QString::number(dev->testData.errcAVG.cd.e32, 'f', 2) );

#ifdef ENABLE_UNCR
		l_ut->setText( QString::number(dev->testData.errcTOT.cd.uncr) );
		l_um->setNum( dev->testData.errcMAX.cd.uncr );
		l_ua->setText( QString::number(dev->testData.errcAVG.cd.uncr, 'f', 2) );
#endif
	} else {
		l_e0t->setText( QString::number(dev->testData.errcTOT.dvd.pi8) );
		l_e0m->setNum( dev->testData.errcMAX.dvd.pi8 );
		l_e0a->setText( QString::number(dev->testData.errcAVG.dvd.pi8, 'f', 2) );

		l_e1t->setText( QString::number(dev->testData.errcTOT.dvd.pif) );
		l_e1m->setNum( dev->testData.errcMAX.dvd.pif );
		l_e1a->setText( QString::number(dev->testData.errcAVG.dvd.pif, 'f', 2) );

		l_e2t->setText( QString::number(dev->testData.errcTOT.dvd.pof) );
		l_e2m->setNum( dev->testData.errcMAX.dvd.pof );
		l_e2a->setText( QString::number(dev->testData.errcAVG.dvd.pof, 'f', 2) );

#ifdef ENABLE_UNCR
		l_ut->setText( QString::number(dev->testData.errcTOT.dvd.uncr) );
		l_um->setNum( dev->testData.errcMAX.dvd.uncr );
		l_ua->setText( QString::number(dev->testData.errcAVG.dvd.uncr, 'f', 2) );
#endif
	}
}

void tabERRC::toggleXErrc()
{
	if (!xerrc) {
		xerrc = new ErrcDetailedDialog(settings,devices,this);
		QObject::connect(xerrc, SIGNAL(closed()), this, SLOT(XerrcClosed()));
		xerrc->show();
		xerrc->updateAll();
	} else {
		xerrc->hide();
		delete xerrc;
		xerrc = NULL;
	}
}

void tabERRC::XerrcClosed()
{
	xerrc->disconnect();
	xerrc->deleteLater();
	xerrc = NULL;
}

