/*
 * This file is part of the QPxTool project.
 * Copyright (C) 2008-2009 Gennady "ShultZ" Kozlov <qpxtool@mail.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 *
 */

#include <QVBoxLayout>
#include <QLabel>

#include <qpxgraph.h>
#include <device.h>
#include "tab_fete.h"
#include <QDebug>

tabFETE::tabFETE(QPxSettings *iset, devlist *idev, QString iname, QWidget *p, Qt::WindowFlags fl)
	: GraphTab(iset, idev, iname, TEST_FT, p, fl)
{
#ifndef QT_NO_DEBUG
	qDebug("STA: tabFETE()");
#endif
	layout_info = new QVBoxLayout(infow);
	layout_info->setMargin(0);
	layout_info->setSpacing(3);

	pl_fmax = new QLabel(tr("FE max"), infow);
	pl_fmax->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_fmax->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	pl_fmax->setMinimumSize(80,22);
	layout_info->addWidget(pl_fmax);
	l_fmax = new QLabel(infow);
	l_fmax->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_fmax->setMinimumSize(80,22);
	layout_info->addWidget(l_fmax);

#ifdef FT_AVG
	pl_favg = new QLabel(tr("FE avg"), infow);
	pl_favg->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_favg->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	pl_favg->setMinimumSize(80,22);
	layout_info->addWidget(pl_favg);
	l_favg = new QLabel(infow);
	l_favg->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_favg->setMinimumSize(80,22);
	layout_info->addWidget(l_favg);
#endif

	pl_tmax = new QLabel(tr("TE max"), infow);
	pl_tmax->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_tmax->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	pl_tmax->setMinimumSize(80,22);
	layout_info->addWidget(pl_tmax);
	l_tmax = new QLabel(infow);
	l_tmax->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_tmax->setMinimumSize(80,22);
	layout_info->addWidget(l_tmax);

#ifdef FT_AVG
	pl_tavg = new QLabel(tr("TE avg"), infow);
	pl_tavg->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	pl_tavg->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
	pl_tavg->setMinimumSize(80,22);
	layout_info->addWidget(pl_tavg);
	l_tavg = new QLabel(infow);
	l_tavg->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	l_tavg->setMinimumSize(80,22);
	layout_info->addWidget(l_tavg);
#endif

	layout_info->addStretch(10);

	clear();
#ifndef QT_NO_DEBUG
	qDebug("END: tabFETE()");
#endif
}

tabFETE::~tabFETE()
{
#ifndef QT_NO_DEBUG
	qDebug("STA: ~tabFETE()");
	qDebug("END: ~tabFETE()");
#endif
}
/*
void tabFETE::clear()
{
	qDebug("tabFETE::clear()");

}
*/
void tabFETE::selectDevice()
{
#ifndef QT_NO_DEBUG
	qDebug("tabFETE::selectDevice()");
#endif
	device *dev = devices->current();
	GraphTab::updateLast((int)dev->testData.ft_time, NULL, 1);
	updateSummary();

	QObject::connect( dev, SIGNAL(doneMInfo(int)), this, SLOT(updateAll()) );
	QObject::connect( dev, SIGNAL(block_FT()), this, SLOT(updateLast()) );
}

void tabFETE::updateLast()
{
	bool show;
	device *dev = devices->current();
	GraphTab::updateLast((int)dev->testData.ft_time, &show);
	if (!show) return;
	updateSummary();
}

void tabFETE::updateAll()
{
	GraphTab::updateLast((int)devices->current()->testData.ft_time, NULL, 1);
	updateSummary();
}

void tabFETE::updateSummary()
{
	device *dev = devices->current();
	if (!dev->testData.jb.size()) {
		l_fmax->clear(); l_tmax->clear();
#ifdef FT_AVG
		l_favg->clear(); l_tavg->clear();
#endif
		return;
	}
// setting values...
	l_fmax->setText(QString::number(dev->testData.ftMAX.fe,'f',2));
	l_tmax->setText(QString::number(dev->testData.ftMAX.te,'f',2));
#ifdef FT_AVG
	l_favg->setText(QString::number(dev->testData.ftMAX.favg,'f',2));
	l_tavg->setText(QString::number(dev->testData.ftMAX.tavg,'f',2));
#endif
}

